package Allegory.PD2;

import PrologDB.DB;
import PrologDB.Table;
import PrologDB.ErrorReport;
import java.io.PrintStream;

public class Database {

    public PersonTable Person = PersonTable.BaseRelation(this);
    public DogTable Dog = DogTable.BaseRelation(this);
    public EntityTable Entity = EntityTable.BaseRelation(this);

    private DB db;

    public Database() {}

    public Database(String filename) {
        // Step 1: Read the database, throw exception if errors found and halt
        db = DB.read(filename);

        // Step 2: access MDELite tables
        Table _Person = db.getTableEH("Person");
        Table _Dog = db.getTableEH("Dog");
        Table _Entity = db.getTableEH("Entity");

        // Step 3: pass1 -- load primitive data of all tables
        _Person.tuplesLocal().forEach(t->Person.makeAndAdd(t));
        _Dog.tuplesLocal().forEach(t->Dog.makeAndAdd(t));
        _Entity.tuplesLocal().forEach(t->Entity.makeAndAdd(t));

        // Step 4: pass2 -- now replace MDELite tuple ids with Java object ids
        ErrorReport er = new ErrorReport();
        Person.tuplesLocal().forEach(t->t.finishTuple(this,er));
        Dog.tuplesLocal().forEach(t->t.finishTuple(this,er));
        Entity.tuplesLocal().forEach(t->t.finishTuple(this,er));
        er.printEH(System.out);
    }

    public void print() { print(System.out); }

    public void prettyprint() { prettyprint(System.out); }

    public void prettyprint(PrintStream out) {
        out.format("dbase(PD2,[Person,Dog,Entity]).\n\n");
        Person.prettyprintLocal(out);
        Dog.prettyprintLocal(out);
        Entity.prettyprintLocal(out);
        out.format("subtable(Entity,[Person,Dog]).\n");
    }

    public void print(PrintStream out) {
        out.format("dbase(PD2,[Person,Dog,Entity]).\n\n");
        Person.printLocal(out);
        Dog.printLocal(out);
        Entity.printLocal(out);
        out.format("subtable(Entity,[Person,Dog]).\n");
    }

}
