package Allegory.PD2;

import PrologDB.Tuple;
import java.io.PrintStream;
import java.util.LinkedList;
import Meta4Framework.TaBle;
import java.util.function.Function;
import static Allegory.PD2.Lang.*;

public class EntityTable extends TaBle<Database,Entity,EntityTable> {
    @Override
    public String TupleTypeName() { return "Entity"; }

    public <TABLE> TABLE applyEntity(Function<EntityTable,TABLE> f) { return f.apply(this); }

    // constructors for creating empty or singleton tables
    public EntityTable(Database db) {
        this(db,null);
    }

    public EntityTable(Database db, Entity t) {
        this.db = db;
        initTupleList(t);
    }

    // methods for loading a database
    static EntityTable BaseRelation(Database db) {
        EntityTable tab = new EntityTable(db);
        tab.computedTable = false;
        return tab;
    }

    @Override
    public LinkedList<Entity> tuples() {
        if (computedTable)
            return tuples;
        LinkedList<Entity> all = new LinkedList<>();
        all.addAll(tuples);
        all.addAll(db.Person.tuples());
        all.addAll(db.Dog.tuples());
        return all;
    }

    @Override
    public EntityTable New() { return new EntityTable(db); }

    @Override
    public void makeAndAdd(Tuple t) {
       Entity r = new Entity(t.get("id"),t.get("name"),t.get("zipcode"),t.get("state"));
       add(r);
    }

    @Override
    public boolean deleteFromSubTables(Entity t) {
       if (computedTable) return false;
       if (t instanceof Person) return db.Person.delete((Person) t);
       if (t instanceof Dog) return db.Dog.delete((Dog) t);
       return false;
    }

    @Override
    public void cascadingDeleteLocal(Entity t) {
    }

    @Override
    public void mastHead(PrintStream out) {
        out.println("table(Entity,[id,name,zipcode,state]).");
    }

    @Override
    public String pformat() { return Entity.fmt; }

    @Override
    public String ppformat(boolean local) {
       int  f0=0, f1=0, f2=0, f3=0;
       for (Entity a : local?tuples:tuples()) {
          f0 = Math.max(f0,a.id.length());
          f1 = Math.max(f1,(""+a.name).length()+2);
          f2 = Math.max(f2,(""+a.zipcode).length()+2);
          f3 = Math.max(f3,(""+a.state).length()+2);
       }
       return String.format("Entity(!%ds,!%ds,!%ds,!%ds).\n",f0,f1,f2,f3).replace("!","%");
    }

    public STRINGTable id() {
       STRINGTable st = new STRINGTable(db);
       tuples.forEach(t -> st.add(new STRING(t.id)));
       return st;
    }

    public STRINGTable name() {
       STRINGTable st = new STRINGTable(db);
       tuples.forEach(t -> st.add(new STRING(t.name)));
       return st;
    }

    public STRINGTable zipcode() {
       STRINGTable st = new STRINGTable(db);
       tuples.forEach(t -> st.add(new STRING(t.zipcode)));
       return st;
    }

    public STRINGTable state() {
       STRINGTable st = new STRINGTable(db);
       tuples.forEach(t -> st.add(new STRING(t.state)));
       return st;
    }

   /* the following methods cast a subtable into a supertable */

   /* the following methods filter a supertable into a subtable */

   public PersonTable toPersonTable() {
      PersonTable tb = new PersonTable(db);
      for (Entity t : tuples()){ if (t instanceof Person) tb.add((Person) t); }
      return tb;
   }

   public DogTable toDogTable() {
      DogTable tb = new DogTable(db);
      for (Entity t : tuples()){ if (t instanceof Dog) tb.add((Dog) t); }
      return tb;
   }

   /* end of filtering methods */

}
